@@ -89,6 +89,7 @@ function show_waiter(id, html, color, to_hide){
   overlay.style.position = "absolute";
   overlay.style.zIndex = 999;
   overlay.classList.add("waiter-overlay");
+  //overlay.style.animation = "expand .15s ease-in-out";
 
   // append overlay content in overlay
   overlay.appendChild(overlay_content);
@@ -104,10 +105,14 @@ function hide_waiter(id){
 
   var overlay = dom.getElementsByClassName("waiter-overlay");
 
-  if(overlay.length > 0)
-    dom.removeChild(overlay[0]);
-  else
+  if(overlay.length > 0){
+    //dom.style.animation = "shrink .15s ease-in-out";
+    setTimeout(function(){
+      dom.removeChild(overlay[0]);
+    }, 150)
+  } else{
     console.log("no waiter on", id);
+  }
 
 }
 
@@ -138,8 +143,8 @@ function hide_recalculate(id){
   head.appendChild(style);
 }
 
-$(document).on('shiny:value shiny:error shiny:recalculated shiny:outputinvalidated', function(event) {
-  if(waiter_to_hide.includes(event.name)){
+$(document).on('shiny:value shiny:error shiny:outputinvalidated', function(event) {
+  if(waiter_to_hide.indexOf(event.name) > 0){
     hide_waiter(event.name);
   }
-});
\ No newline at end of file
+});
